function plot_benchmark_final()


clearvars;
clc;
%close all;

data = xlsread('3_inch_3_phase_benchmark_normalised_dd',1);

nom_wc    = 100*data(:,1);
nom_mf    = data(:,2);
ref_wc    = data(:,3);
ref_mf    = data(:,4);
ref_dens  = data(:,5);
ref_GVF   = data(:,6);
raw_mf    = data(:,7);
raw_dens  = data(:,8);
raw_dd    = data(:,9);
err_mf    = data(:,10);
err_dens  = data(:,11);
err_dd    = data(:,12);

% Determine nominal water cuts and first and last indices for each

n_wc = 1;
data_index = 1;
data_length = length(nom_wc);

wc_set{n_wc}.start = data_index;
wc_set{n_wc}.end = data_length;
wc_set{n_wc}.wc = nom_wc(data_index);

while (data_index < data_length)
    data_index = data_index+1;
    if (nom_wc(data_index) ~= wc_set{n_wc}.wc) % new water cut set
        wc_set{n_wc}.end = data_index - 1;
        n_wc = n_wc+1;
        
        wc_set{n_wc}.start = data_index;
        wc_set{n_wc}.end = data_length;
        wc_set{n_wc}.wc = nom_wc(data_index);        
    end
end

% Now, for each water cut, find each line

min_mf = 1e6;
max_mf = -1e6;

for i = 1:n_wc
    data_index = wc_set{i}.start;
    n_lines = 1;
    wc_set{i}.n_lines = n_lines;
    wc_set{i}.lines{n_lines}.start = wc_set{i}.start;
    wc_set{i}.lines{n_lines}.end   = wc_set{i}.end;
    wc_set{i}.lines{n_lines}.flow  = nom_mf(data_index);

    while (data_index < wc_set{i}.end)
        
        if (nom_mf(data_index) < min_mf)
            min_mf = nom_mf(data_index);
        end
        
        if (nom_mf(data_index) > max_mf)
            max_mf = nom_mf(data_index);
        end
                
        data_index = data_index+1;
        if (nom_mf(data_index) ~= wc_set{i}.lines{n_lines}.flow) % new flow line
            wc_set{i}.lines{n_lines}.end = data_index - 1;
        
            n_lines = n_lines+1;
            wc_set{i}.n_lines = n_lines;
            
            wc_set{i}.lines{n_lines}.start = data_index;
            wc_set{i}.lines{n_lines}.end = wc_set{i}.end;
            wc_set{i}.lines{n_lines}.flow  = nom_mf(data_index);
                        
        end
    end
end


% Do plots for each water cut

plot_str = {'g-*',  'g-o',  'g-x',  'g-s',  'g-d',  'g-^',  'g-p',  'g-h', ...
            'b--*', 'b--o', 'b--x', 'b--s', 'b--d', 'b--^', 'b--p', 'b--h'};

n_plot_str = 16;

ydiff = (max_mf-min_mf);

extend_min_mf = min_mf - ydiff/10;
extend_max_mf = max_mf + ydiff/10;

if (1) % plot results
    
    for i = 1: n_wc
        
        k = wc_set{i}.n_lines;
        
        figure_start = 10*i;
        
        %% plot reference measurements
        
        fig_no = figure_start + 1;
        figure(fig_no);
        
        leg_strs = cell(1,k);
        
        eval_str = 'legend(';
        
        for j = 1:k
            plot_ind = round(n_plot_str*j/k);
            n1 = wc_set{i}.lines{j}.start;
            n2 = wc_set{i}.lines{j}.end;
            plot(ref_GVF(n1:n2), ref_mf(n1:n2), plot_str{plot_ind});
            hold on
            leg_strs{j} = sprintf('%f4.1 kg/s',wc_set{i}.lines{j}.flow);
            
            if (j < k)
                eval_str = sprintf('%s''%0.1f kg/s'', ', eval_str, wc_set{i}.lines{j}.flow);
            else
                eval_str = sprintf('%s''%0.1f kg/s'', ''Location'', ''northeastoutside'');', eval_str, wc_set{i}.lines{j}.flow);
            end
        end
        hold off
        grid
        eval(eval_str)
        xlabel('GVF (%)');
        ylabel('Mass flow rate (kg/s)');
        title_str = sprintf('WC = %d%%. Reference flow rate and GVF.', wc_set{i}.wc);
        title(title_str);
        ylim([extend_min_mf, extend_max_mf]);
        xlim([0,75]);
        
        plot_name = sprintf('%03d_WC%03d_ref_meas', fig_no, wc_set{i}.wc);
        do_plot(plot_name);
        
        %% plot mass flow errors vs dd
        
        fig_no = figure_start + 2;
        figure(fig_no);
        
        leg_strs = cell(1,k);
        
        eval_str = 'legend(';
        
        for j = 1:k
            plot_ind = round(n_plot_str*j/k);
            n1 = wc_set{i}.lines{j}.start;
            n2 = wc_set{i}.lines{j}.end;
            plot(raw_dd(n1:n2), err_mf(n1:n2), plot_str{plot_ind});
            hold on
            leg_strs{j} = sprintf('%f4.1 kg/s',wc_set{i}.lines{j}.flow);
            
            if (j < k)
                eval_str = sprintf('%s''%0.1f kg/s'', ', eval_str, wc_set{i}.lines{j}.flow);
            else
                eval_str = sprintf('%s''%0.1f kg/s'', ''Location'', ''northeastoutside'');', eval_str, wc_set{i}.lines{j}.flow);
            end
        end
        hold off
        grid
        eval(eval_str)
        xlabel('Density drop (%)');
        ylabel('Mass flow error (%)');
        title_str = sprintf('WC = %d%%. Mass flow error vs density drop.', wc_set{i}.wc);
        title(title_str);
        ylim([-45, 5]);
        xlim([0,80]);
        
        plot_name = sprintf('%03d_WC%03d_mf_err', fig_no, wc_set{i}.wc);
        do_plot(plot_name);
        
        %% plot density drop errors vs dd
        
        fig_no = figure_start + 3;
        figure(fig_no);
        
        leg_strs = cell(1,k);
        
        eval_str = 'legend(';
        
        for j = 1:k
            plot_ind = round(n_plot_str*j/k);
            n1 = wc_set{i}.lines{j}.start;
            n2 = wc_set{i}.lines{j}.end;
            plot(raw_dd(n1:n2), err_dd(n1:n2), plot_str{plot_ind});
            hold on
            leg_strs{j} = sprintf('%f4.1 kg/s',wc_set{i}.lines{j}.flow);
            
            if (j < k)
                eval_str = sprintf('%s''%0.1f kg/s'', ', eval_str, wc_set{i}.lines{j}.flow);
            else
                eval_str = sprintf('%s''%0.1f kg/s'', ''Location'', ''northeastoutside'');', eval_str, wc_set{i}.lines{j}.flow);
            end
        end
        hold off
        grid
        eval(eval_str)
        xlabel('Density drop (%)');
        ylabel('Density drop error (%)');
        title_str = sprintf('WC = %d%%. Density drop error vs density drop.', wc_set{i}.wc);
        title(title_str);
        ylim([-10, 30]);
        xlim([0,80]);
        
        plot_name = sprintf('%03d_WC%03d_dd_err', fig_no, wc_set{i}.wc);
        do_plot(plot_name);
        
    end
    
end

return






